@@ -9,6 +9,8 @@ struct auth_master_connection {
 	struct auth *auth;
 	int refcount;
 
+	struct timeval create_time, handshake_time;
+
 	int fd;
 	char *path;
 	struct istream *input;
@@ -25,6 +27,9 @@ struct auth_master_connection {
 	unsigned int userdb_only:1;
 };
 
+void auth_master_log_error(struct auth_master_connection *conn,
+			   const char *fmt, ...) ATTR_FORMAT(2, 3);
+
 struct auth_master_connection *
 auth_master_connection_create(struct auth *auth, int fd,
 			      const char *path, const struct stat *socket_st,
@@ -34,7 +39,7 @@ void auth_master_connection_destroy(struct auth_master_connection **conn);
 void auth_master_connection_ref(struct auth_master_connection *conn);
 void auth_master_connection_unref(struct auth_master_connection **conn);
 
-void auth_master_request_callback(const char *reply, void *context);
+void auth_master_request_callback(const char *reply, struct auth_master_connection *conn);
 
 void auth_master_connections_destroy_all(void);
 
