@@ -19,13 +19,17 @@
 	(sizeof(arr) / sizeof((arr)[0]))
 
 #define MEM_ALIGN(size) \
-	(((size) + MEM_ALIGN_SIZE-1) & ~((unsigned int) MEM_ALIGN_SIZE-1))
+	(((size) + MEM_ALIGN_SIZE-1) & ~((size_t) MEM_ALIGN_SIZE-1))
 
 #define PTR_OFFSET(ptr, offset) \
 	((void *) (((unsigned char *) (ptr)) + (offset)))
 #define CONST_PTR_OFFSET(ptr, offset) \
 	((const void *) (((const unsigned char *) (ptr)) + (offset)))
 
+#define container_of(ptr, type, name) \
+	(type *)((uintptr_t)(ptr) - (uintptr_t)offsetof(type, name) + \
+		 COMPILE_ERROR_IF_TYPES_NOT_COMPATIBLE(ptr, &((type *) 0)->name))
+
 /* Don't use simply MIN/MAX, as they're often defined elsewhere in include
    files that are included after this file generating tons of warnings. */
 #define I_MIN(a, b)  (((a) < (b)) ? (a) : (b))
@@ -109,6 +113,16 @@
 #  define ATTR_CONST
 #  define ATTR_PURE
 #endif
+#ifdef HAVE_ATTR_NULL
+#  define ATTR_NULL(...) __attribute__((null(__VA_ARGS__)))
+#else
+#  define ATTR_NULL(...)
+#endif
+#ifdef HAVE_ATTR_NOWARN_UNUSED_RESULT
+#  define ATTR_NOWARN_UNUSED_RESULT __attribute__((nowarn_unused_result))
+#else
+#  define ATTR_NOWARN_UNUSED_RESULT
+#endif
 #if __GNUC__ > 2
 #  define ATTR_MALLOC __attribute__((malloc))
 #else
@@ -130,35 +144,40 @@
 #  define ATTR_HOT
 #  define ATTR_COLD
 #endif
+#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 9)
+/* GCC 4.9 and later */
+#  define ATTR_RETURNS_NONNULL __attribute__((returns_nonnull))
+#else
+#  define ATTR_RETURNS_NONNULL
+#endif
 
 /* Macros to provide type safety for callback functions' context parameters */
-#ifdef __GNUC__
-#  define CONTEXT_TYPE_SAFETY
-#endif
-#ifdef CONTEXT_TYPE_SAFETY
-#  define CONTEXT_CALLBACK(name, callback_type, callback, context, ...) \
-	({(void)(1 ? 0 : callback(context)); \
-	name(__VA_ARGS__, (callback_type *)callback, context); })
-#  define CONTEXT_CALLBACK2(name, callback_type, callback, arg1_type, context, ...) \
-	({(void)(1 ? 0 : callback((arg1_type)0, context)); \
-	name(__VA_ARGS__, (callback_type *)callback, context); })
+#ifdef HAVE_TYPE_CHECKS
+#  define CALLBACK_TYPECHECK(callback, type) \
+	(COMPILE_ERROR_IF_TRUE(!__builtin_types_compatible_p( \
+		typeof(&callback), type)) ? 1 : 0)
 #else
-#  define CONTEXT_CALLBACK(name, callback_type, callback, context, ...) \
-	name(__VA_ARGS__, (callback_type *)callback, context)
-#  define CONTEXT_CALLBACK2(name, callback_type, callback, arg1_type, context, ...) \
-	name(__VA_ARGS__, (callback_type *)callback, context)
+#  define CALLBACK_TYPECHECK(callback, type) 0
 #endif
 
-#if __GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ > 0)
-#  define HAVE_TYPEOF
+#if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ > 0)) && !defined(__cplusplus)
 #  define COMPILE_ERROR_IF_TRUE(condition) \
-	(sizeof(char[1 - 2 * !!(condition)]) - 1)
+	(sizeof(char[1 - 2 * ((condition) ? 1 : 0)]) - 1)
+#else
+#  define COMPILE_ERROR_IF_TRUE(condition) 0
+#endif
+
+#ifdef HAVE_TYPE_CHECKS
 #  define COMPILE_ERROR_IF_TYPES_NOT_COMPATIBLE(_a, _b) \
 	COMPILE_ERROR_IF_TRUE( \
 		!__builtin_types_compatible_p(typeof(_a), typeof(_b)))
+#define COMPILE_ERROR_IF_TYPES2_NOT_COMPATIBLE(_a1, _a2, _b) \
+	COMPILE_ERROR_IF_TRUE( \
+		!__builtin_types_compatible_p(typeof(_a1), typeof(_b)) && \
+		!__builtin_types_compatible_p(typeof(_a2), typeof(_b)))
 #else
-#  define COMPILE_ERROR_IF_TRUE(condition) 0
 #  define COMPILE_ERROR_IF_TYPES_NOT_COMPATIBLE(_a, _b) 0
+#  define COMPILE_ERROR_IF_TYPES2_NOT_COMPATIBLE(_a1, _a2, _b) 0
 #endif
 
 #if __GNUC__ > 2
@@ -169,6 +188,12 @@
 #  define likely(expr) expr
 #endif
 
+#if defined(__clang__) && ((__clang_major__ > 4) || (__clang_major__ == 3 && __clang_minor__ >= 9))
+#  define ATTR_UNSIGNED_WRAPS __attribute__((no_sanitize("integer")))
+#else
+#  define ATTR_UNSIGNED_WRAPS
+#endif
+
 /* Provide macros for error handling. */
 #ifdef DISABLE_ASSERTS
 #  define i_assert(expr)
@@ -193,6 +218,16 @@
 
 #endif
 
+/* Close the fd and set it to -1. This assert-crashes if fd == 0. Normally
+   this would happen only if an uninitialized fd is attempted to be closed,
+   which is a bug. */
+#define i_close_fd(fd) STMT_START {  \
+	i_assert(*fd > 0); \
+	if (unlikely(close_keep_errno(fd) < 0)) \
+		i_error("close(%d[%s:%d]) failed: %m", \
+			*(fd), __FILE__, __LINE__); \
+	} STMT_END
+
 #define i_unreached() \
 	i_panic("file %s: line %d: unreached", __FILE__, __LINE__)
 
@@ -204,4 +239,13 @@
 #  define DOVECOT_PREREQ(maj, min) 0
 #endif
 
+#ifdef __cplusplus
+#  undef STATIC_ARRAY
+#  define STATIC_ARRAY
+#endif
+
+/* Convenience wrappers for initializing a struct */
+#define i_zero(p) memset(p, 0, sizeof(*(p)))
+#define i_zero_safe(p) safe_memset(p, 0, sizeof(*(p)))
+
 #endif
