@@ -5,10 +5,13 @@
 
 struct mail_storage_callbacks;
 
-enum namespace_type {
-	NAMESPACE_PRIVATE	= 0x01,
-	NAMESPACE_SHARED	= 0x02,
-	NAMESPACE_PUBLIC	= 0x04
+enum mail_namespace_type {
+	MAIL_NAMESPACE_TYPE_PRIVATE	= 0x01,
+	MAIL_NAMESPACE_TYPE_SHARED	= 0x02,
+	MAIL_NAMESPACE_TYPE_PUBLIC	= 0x04
+#define MAIL_NAMESPACE_TYPE_MASK_ALL \
+	(MAIL_NAMESPACE_TYPE_PRIVATE | MAIL_NAMESPACE_TYPE_SHARED | \
+	 MAIL_NAMESPACE_TYPE_PUBLIC)
 };
 
 enum namespace_flags {
@@ -46,7 +49,7 @@ struct mail_namespace {
 	struct mail_namespace *next;
 	int refcount;
 
-        enum namespace_type type;
+        enum mail_namespace_type type;
 	enum namespace_flags flags;
 
 	char *prefix;
@@ -64,23 +67,59 @@ struct mail_namespace {
 
 	struct mail_user *user, *owner;
 	struct mailbox_list *list;
-	/* FIXME: we should support multiple storages in one namespace */
-	struct mail_storage *storage;
+	struct mail_storage *storage; /* default storage */
+	ARRAY(struct mail_storage *) all_storages;
 
-	struct mail_namespace_settings *set, *unexpanded_set;
+	/* This may point to user->set, but it may also point to
+	   namespace-specific settings. When accessing namespace-specific
+	   settings it should be done through here instead of through the
+	   mail_user. */
+	struct mail_user_settings *user_set;
+
+	const struct mail_namespace_settings *set, *unexpanded_set;
 	const struct mail_storage_settings *mail_set;
 
+	unsigned int special_use_mailboxes:1;
 	unsigned int destroyed:1;
 };
 
+
+/* Allocate a new namespace, and fill it based on the passed in settings.
+   This is the most low-level namespace creation function. The storage isn't
+   initialized for the namespace.
+
+   user_all_settings normally points to user->set. If you want to override
+   settings for the created namespace, you can duplicate the user's settings
+   and provide a pointer to it here. Note that the pointer must contain
+   ALL the settings, including the dynamic driver-specific settings, so it
+   needs to created via settings-parser API. */
+int mail_namespace_alloc(struct mail_user *user,
+			 void *user_all_settings,
+			 struct mail_namespace_settings *ns_set,
+			 struct mail_namespace_settings *unexpanded_set,
+			 struct mail_namespace **ns_r,
+			 const char **error_r);
+
+/* Add and initialize namespaces to user based on namespace settings. */
 int mail_namespaces_init(struct mail_user *user, const char **error_r);
+/* Add and initialize INBOX namespace to user based on the given location. */
 int mail_namespaces_init_location(struct mail_user *user, const char *location,
-				  const char **error_r);
+				  const char **error_r) ATTR_NULL(2);
+/* Add an empty namespace to user. */
 struct mail_namespace *mail_namespaces_init_empty(struct mail_user *user);
 /* Deinitialize all namespaces. mail_user_deinit() calls this automatically
    for user's namespaces. */
 void mail_namespaces_deinit(struct mail_namespace **namespaces);
 
+/* Allocate a new namespace and initialize it. This is called automatically by
+   mail_namespaces_init(). */
+int mail_namespaces_init_add(struct mail_user *user,
+			     struct mail_namespace_settings *ns_set,
+			     struct mail_namespace_settings *unexpanded_ns_set,
+			     struct mail_namespace **ns_p, const char **error_r);
+int mail_namespaces_init_finish(struct mail_namespace *namespaces,
+				const char **error_r);
+
 void mail_namespace_ref(struct mail_namespace *ns);
 void mail_namespace_unref(struct mail_namespace **ns);
 
@@ -105,29 +144,32 @@ char mail_namespace_get_sep(struct mail_namespace *ns);
 char mail_namespaces_get_root_sep(struct mail_namespace *namespaces)
 	ATTR_PURE;
 
-/* Returns namespace based on the mailbox name's prefix, or NULL if no matching
-   namespace could be found. */
+/* Returns namespace based on the mailbox name's prefix. Note that there is
+   always a prefix="" namespace, so for this function NULL is never returned. */
 struct mail_namespace *
 mail_namespace_find(struct mail_namespace *namespaces, const char *mailbox);
-/* Find namespace for mailbox and return it. If the namespace has alias_for
-   set, return that namespace instead and change mailbox name to be a valid
+/* Same as mail_namespace_find(), but if the namespace has alias_for set,
+   return that namespace instead and change mailbox name to be a valid
    inside it. */
 struct mail_namespace *
 mail_namespace_find_unalias(struct mail_namespace *namespaces,
 			    const char **mailbox);
-/* Like above, but ignore hidden namespaces. */
+
+/* Like mail_namespace_find(), but ignore hidden namespaces. */
 struct mail_namespace *
 mail_namespace_find_visible(struct mail_namespace *namespaces,
 			    const char *mailbox);
-/* Like above, but find only from namespaces with subscriptions flag set. */
+/* Like mail_namespace_find(), but find only from namespaces with
+   subscriptions=yes. */
 struct mail_namespace *
 mail_namespace_find_subscribable(struct mail_namespace *namespaces,
 				 const char *mailbox);
-/* Like above, but find only from namespaces with subscriptions flag not set. */
+/* Like mail_namespace_find(), but find only from namespaces with
+   subscriptions=no. */
 struct mail_namespace *
 mail_namespace_find_unsubscribable(struct mail_namespace *namespaces,
 				   const char *mailbox);
-/* Returns the INBOX namespace */
+/* Returns the INBOX namespace. It always exists, so NULL is never returned. */
 struct mail_namespace *
 mail_namespace_find_inbox(struct mail_namespace *namespaces);
 /* Find a namespace with given prefix. */
@@ -143,4 +185,21 @@ mail_namespace_find_prefix_nosep(struct mail_namespace *namespaces,
 void mail_namespace_finish_list_init(struct mail_namespace *ns,
 				     struct mailbox_list *list);
 
+/* Returns TRUE if this is the root of a type=shared namespace that is actually
+   used for accessing shared users' mailboxes (as opposed to marking a
+   type=public namespace "wrong"). */
+bool mail_namespace_is_shared_user_root(struct mail_namespace *ns);
+
+/* Returns TRUE if namespace includes INBOX that should be \Noinferiors.
+   This happens when the namespace has a prefix, which is not empty and not
+   "INBOX". This happens, because if storage_name=INBOX/foo it would be
+   converted to vname=prefix/INBOX/foo. */
+static inline bool
+mail_namespace_is_inbox_noinferiors(struct mail_namespace *ns)
+{
+	return (ns->flags & NAMESPACE_FLAG_INBOX_USER) != 0 &&
+		ns->prefix_len > 0 &&
+		strncmp(ns->prefix, "INBOX", ns->prefix_len-1) != 0;
+}
+
 #endif
